@@ -161,6 +161,9 @@ release_metadata (GVfsBackendCdda *cdda_backend)
 static char *
 cdtext_string_to_utf8 (const char *string)
 {
+  if (string == NULL)
+    return NULL;
+
   /* CD-text doesn't specify encoding. In case outside ascii, assume latin-1. */
   return g_convert (string, -1, "UTF-8", "ISO-8859-1", NULL, NULL, NULL);
 }
@@ -545,7 +548,7 @@ do_unmount (GVfsBackend *backend,
 
 /* returns -1 if we couldn't map */
 static int
-get_track_num_from_name (GVfsBackendCdda *cdda_backend, const char *filename)
+get_track_num_from_name (const char *filename)
 {
   int n;
   char *basename;
@@ -715,7 +718,7 @@ do_open_for_read (GVfsBackend *backend,
 
   read_handle = g_new0 (ReadHandle, 1);
 
-  track_num = get_track_num_from_name (cdda_backend, job->filename);
+  track_num = get_track_num_from_name (job->filename);
   if (track_num == -1)
     {
       error = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, 
@@ -999,10 +1002,18 @@ do_query_info (GVfsBackend *backend,
     }
   else
     {
-      g_file_info_set_name (info, filename);
-      g_file_info_set_display_name (info, filename);
-      
-      track_num = get_track_num_from_name (cdda_backend, filename);
+      if (*filename == '/')
+        {
+          g_file_info_set_name (info, filename + 1);
+          g_file_info_set_display_name (info, filename + 1);
+        }
+      else
+        {
+          g_file_info_set_name (info, filename);
+          g_file_info_set_display_name (info, filename);
+        }
+
+      track_num = get_track_num_from_name (filename);
       if (track_num == -1)
         {
           error = g_error_new (G_IO_ERROR, G_IO_ERROR_NOT_FOUND, _("No such file"));
